/*
 * MainFrame.java
 *
 * Created on May 17, 2011, 8:58:59 PM
 */

package gui;

import java.util.EventListener;
import java.util.List;
import javax.swing.SwingWorker;

/**
 * Application to demonstrate the use of a {@code SwingWorker} to run
 * non-GUI code in a separate thread, and how to use the {@code publish}
 * and {@code process} methods to communicate data from the worker thread
 * to the GUI thread.
 * <p>
 * Note that the functionality of this application could be (better) achieved
 * without a background thread, by using {@code javax.swing.Timer}.
 * Imagine, however, that the background calculation involves
 * long-running functionality.
 *
 * @author Tom Verhoeff (TU/e, Eindhoven University of Technology)
 */
public class MainFrame extends javax.swing.JFrame {

    /** Creates new form MainFrame. */
    public MainFrame() {
        initComponents();
        initFrame();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jTextField1 = new javax.swing.JTextField();
        jButtonStart = new javax.swing.JButton();
        jButtonReset = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Simple Background Calculation");

        jTextField1.setColumns(5);
        jTextField1.setEditable(false);
        jTextField1.setHorizontalAlignment(javax.swing.JTextField.RIGHT);

        jButtonStart.setText("Start");
        jButtonStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStartActionPerformed(evt);
            }
        });

        jButtonReset.setText("Reset");
        jButtonReset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResetActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jButtonStart)
                        .add(18, 18, 18)
                        .add(jButtonReset))
                    .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(209, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(18, 18, 18)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jButtonStart)
                    .add(jButtonReset))
                .addContainerGap(205, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStartActionPerformed
        calculationWorker = new SwingWorker<Void, Integer>() {

            /** The calculation listener that pushes new values to the GUI. */
            private final CalculationListener calculationListener =
                    new CalculationListener() {
                        public void valueCalculated(int value) {
                            publish(value);
                        }
                    };

            /**
             * Execution of the calculation is done in the background thread.
             */
            @Override
            protected Void doInBackground() throws Exception {
                calculate(calculationListener);
                return null;
            }

            /**
             * Called in the GUI thread, in response to calls
             * of publish() in the background thread.
             * Possibly, a sequence of calls to publish() are combined
             * into a single call of process();
             * hence, the List parameter of process().
             */
            @Override
            protected void process(List<Integer> chunks) {
                for (Integer value : chunks) {
                    jTextField1.setText("" + value);
                }
            }

            /**
             * Called in the GUI thread after background thread ends.
             */
            @Override
            protected void done() {
                jTextField1.setText("" + 0);
                updateFrame();
            }
        };
        calculationWorker.execute();
        updateFrame();
    }//GEN-LAST:event_jButtonStartActionPerformed

    private void jButtonResetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResetActionPerformed
        // Just kill the background thread.
        // N.B. This is NOT SAFE if the background thread modifies
        // global data structures!
        calculationWorker.cancel(true);
    }//GEN-LAST:event_jButtonResetActionPerformed

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new MainFrame().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonReset;
    private javax.swing.JButton jButtonStart;
    private javax.swing.JTextField jTextField1;
    // End of variables declaration//GEN-END:variables

    /** The SwingWorker. */
    private SwingWorker<Void, Integer> calculationWorker;

    /** Listener interface for calculations. */
    interface CalculationListener extends EventListener {
        /** Reports a next value that has been calculated. */
        public void valueCalculated(int value);
    }

    /** Complete the initialization of this frame. */
    private void initFrame() {
        jTextField1.setText("0");
        updateFrame();
    }

    /** Update this frame. */
    private void updateFrame() {
        final boolean idle;
        idle = calculationWorker == null || calculationWorker.isDone();
        jButtonStart.setEnabled(idle);
        jButtonReset.setEnabled(! idle);
    }

    /**
     * Calculation to be run in a background thread.
     * It has one listener.
     *
     * For the purpose of this demonstration, the calculation just waits
     * one second and produces the square of the number of seconds elapsed.
     * Imagine that it is doing an important tedious calculation
     * that produces a stream of values to be reported in the GUI.
     *
     * @param listener  the calculation listener
     * @pre {@code listener != null}
     */
    private void calculate(final CalculationListener listener)
            throws InterruptedException {
        int seconds = 0; // number of seconds elapsed since start

        while (true) {
            listener.valueCalculated(seconds * seconds);
            Thread.sleep(1000); // wait 1000 ms == 1 s
            ++ seconds;
        }
    }

}
